/* Copyright (c) 2009, Guillaume Delhumeau & Sophie Poullain
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Guillaume Delhumeau, Sophie Poullain nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package ecosysteme.gui;

import ecosysteme.model.Map;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferStrategy;
import java.awt.image.BufferedImage;

/**
 * Classe fournissant le support sur lequel on dessine la simulation
 * @author Guillaume Delhumeau & Sophie Poullain
 */
public class Display extends Canvas
{

	private BufferStrategy strategy;
	private BufferedImage backBuffer;

	public Display()
	{
	}

	public void init()
	{
		createBufferStrategy(2);
		strategy = getBufferStrategy();
		backBuffer = new BufferedImage(Map.CASE_WIDTH * Map.NB_COLS,
						Map.CASE_HEIGHT * Map.NB_LINES,
						BufferedImage.TYPE_INT_RGB);
	}

	public Graphics getMapGraphics()
	{
		return backBuffer.getGraphics();
	}

	/**
	 * Affiche le buffer courant à l'écran, après mise à échelle
	 */
	public void display()
	{
		Graphics g = strategy.getDrawGraphics();

		g.setColor(Color.DARK_GRAY);
		g.fillRect(0, 0, getWidth(), getHeight());

		int size = Math.min(getWidth(), getHeight());

		int destX = getWidth() / 2 - size / 2;
		int destY = getHeight() / 2 - size / 2;

		g.drawImage(backBuffer,
						destX, destY,
						destX + size,
						destY + size,
						0, 0,
						Map.CASE_WIDTH * Map.NB_COLS,
						Map.CASE_HEIGHT * Map.NB_LINES,
						this);

		strategy.show();
	}
}
