/* Copyright (c) 2009, Guillaume Delhumeau & Sophie Poullain
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Guillaume Delhumeau, Sophie Poullain nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package ecosysteme.gui;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.Range;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * Propose un graphique à placer dans l'interface.
 * @author Guillaume Delhumeau & Sophie Poullain
 */
public class Statistique
{

	private ChartPanel display;
	private XYSeriesCollection datasetFish;
	private XYSeriesCollection datasetShark;
	private XYSeries serieFish;
	private XYSeries serieShark;
	private NumberAxis axis2;
	private int maxShark;
	private JFreeChart chart;

	public Statistique()
	{
		serieFish = new XYSeries("Poissons");
		serieShark = new XYSeries("Requins");

		datasetFish = new XYSeriesCollection();
		datasetFish.addSeries(serieFish);

		datasetShark = new XYSeriesCollection();
		datasetShark.addSeries(serieShark);

		chart = ChartFactory.createXYLineChart(
						"Evolution des populations", // Title   
						"Temps (en nombre d'itérations)",
						"Population de poissons",
						datasetFish, // datasetFish 
						PlotOrientation.VERTICAL,
						true, // Don’t show legend  
						false,
						false);

		axis2 = new NumberAxis("Population de requins");
		XYPlot plot = (XYPlot) chart.getPlot();
		plot.setRangeAxis(1, axis2);
		plot.setRangeAxisLocation(1, AxisLocation.BOTTOM_OR_RIGHT);
		plot.setDataset(1, (XYDataset) datasetShark);
		plot.mapDatasetToRangeAxis(1, 1);
		plot.mapDatasetToDomainAxis(1, 0);

		XYItemRenderer rendu = new XYLineAndShapeRenderer(true, false);
		plot.setRenderer(1, rendu);

		maxShark = 10;
		axis2.setRange(new Range(0, maxShark));
		axis2.setStandardTickUnits(NumberAxis.createIntegerTickUnits());

		display = new ChartPanel(chart);

	}

	public void init()
	{
		serieFish.clear();
		serieShark.clear();
		maxShark = 10;
	}

	public void update(int iter, int nbFish, int nbShark)
	{
		if (iter % 50 == 0)
		{
			serieFish.add(iter, nbFish);
			serieShark.add(iter, nbShark);

			/*if(iter > 3000)
			{
			serieFish.remove(0);
			serieShark.remove(0);
			}*/

			if (nbShark >= maxShark)
			{
				maxShark = nbShark + 1;
			}
			StatistiqueThread thread = new StatistiqueThread(this);
			thread.start();
		}
	}

	public ChartPanel getDisplay()
	{
		return display;
	}

	public void setDisplay(ChartPanel display)
	{
		this.display = display;
	}

	void paint()
	{
		axis2.setRange(new Range(0, maxShark));
		axis2.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
		display.repaint();
	}
}
