/* Copyright (c) 2009, Guillaume Delhumeau & Sophie Poullain
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Guillaume Delhumeau, Sophie Poullain nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package ecosysteme.model;

import java.awt.Color;
import java.awt.Graphics;

/**
 * Une case est le plus petit élément qui compose une carte.
 * @author Guillaume Delhumeau & Sophie Poullain
 */
public class Case
{

	private double planktonQuantity;
	private Fish fish;
	private Shark shark;
	public final static int CASE_WIDTH = 10;
	public final static int CASE_HEIGHT = 10;
	public final static int TYPE_NORMAL = 1;
	public final static int TYPE_PROTECTED = 2;
	private int type;

	public Case()
	{
		planktonQuantity = Math.random() * 100;
		type = TYPE_NORMAL;
	}

	/**
	 * Le type permet de savoir si la case contient un rocher ou non
	 */
	public int getType()
	{
		return type;
	}

	public void setType(int type)
	{
		this.type = type;
	}

	/**
	 * Affiche la case sur le buffer g à la position (x,y).
	 * @param g
	 * @param x
	 * @param y
	 */
	public void draw(Graphics g, int x, int y)
	{
		// Affichage du fond (plancton)
		int c = (int) getPlanktonQuantity();
		if (c > 255)
		{
			c = 255;
		}

		if (type == TYPE_NORMAL)
		{
			g.setColor(new Color(0, 200 - c / 2, 255));
		}
		else
		{
			g.setColor(new Color(200 - c / 2, 200 - c / 2, 200 - c / 2));
		}
		g.fillRect(x, y, CASE_WIDTH, CASE_HEIGHT);

		// Affichage du poisson contenu dans cette case (si besoin)
		if (fish != null)
		{
			fish.draw(g);
		}
	}

	/**
	 * Met à jour la case.
	 * @param iter Itération courante
	 */
	public void update(int iter)
	{
		planktonQuantity += Math.random() * Parameters.PLANKTON_REGENERATION *
						2 - Parameters.PLANKTON_REGENERATION;

		if (planktonQuantity < 0)
		{
			planktonQuantity = 0;
		}
		if (planktonQuantity > 255)
		{
			planktonQuantity = 255;
		}
		// Update des poissons
		if (fish != null)
		{
			fish.update(iter);
		}

		if (shark != null)
		{
			shark.update(iter);
		}
	}

	public double getPlanktonQuantity()
	{
		return planktonQuantity;
	}

	public void setPlanktonQuantity(double planktonQuantity)
	{
		this.planktonQuantity = planktonQuantity;
	}

	public Fish getFish()
	{
		return fish;
	}

	public void setFish(Fish fish)
	{
		this.fish = fish;
	}

	public Shark getShark()
	{
		return shark;
	}

	public void setShark(Shark shark)
	{
		this.shark = shark;
	}
}
