/* Copyright (c) 2009, Guillaume Delhumeau & Sophie Poullain
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Guillaume Delhumeau, Sophie Poullain nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package ecosysteme.model;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;

/**
 * Représente un poisson.
 * @author Guillaume Delhumeau & Sophie Poullain
 */
public class Fish
{

	private static Image sprite;
	private int x;
	private int y;
	private int age;
	private int food;
	private Map map;
	private int lastCopulation;
	private boolean sex;
	public final static boolean SEX_FEMALE = false;
	public final static boolean SEX_MALE = true;
	private long lastIter = 0;

	public Fish(Map map)
	{
		this.map = map;
		lastCopulation = 0;
		if (Math.random() > 0.5)
		{
			sex = SEX_FEMALE;
		}
		else
		{
			sex = SEX_MALE;
		}
		age = 0;
	}

	public void draw(Graphics g)
	{
		g.drawImage(sprite, x * Case.CASE_WIDTH, y * Case.CASE_HEIGHT, null);
	}

	public int getX()
	{
		return x;
	}

	public void setX(int x)
	{
		this.x = x;
	}

	public int getY()
	{
		return y;
	}

	public void setY(int y)
	{
		this.y = y;
	}

	public int getAge()
	{
		return age;
	}

	public void setAge(int age)
	{
		this.age = age;
	}

	public int getFood()
	{
		return food;
	}

	public void setFood(int food)
	{
		this.food = food;
	}

	public void update(int iter)
	{
		// On évite de mettre à jour le poisson plus d'une fois par itération
		if (iter > lastIter)
		{
			move();
			eat();
			tryCopulation();
			getOlder();
			lastIter = iter;
		}
	}

	private void getOlder()
	{
		age++;
		if (age > Parameters.FISH_SHELF_LIFE)
		{
			Case c = map.getCase(x, y);
			c.setFish(null);
		}
		if (food <= 0)
		{
			Case c = map.getCase(x, y);
			c.setFish(null);
		}
		food -= Parameters.FISH_FOOD_QUI_SE_PERD;
	}

	private void tryCopulation()
	{
		if (!canCopulate())
		{
			return;
		}
		// He can do it !!!! But only if he find someone


		int partnerPerimeter = 1;
		// Looking for a partner
		for (int nextX = x - partnerPerimeter; nextX < x + partnerPerimeter; ++nextX)
		{
			for (int nextY = y - partnerPerimeter; nextY < y + partnerPerimeter; ++nextY)
			{
				// On vérfie qu'on a des coordonnées valides
				if (nextX >= 0 && nextX < Map.NB_COLS &&
								nextY >= 0 && nextY < Map.NB_LINES &&
								!(nextX == x && nextY == y))
				{
					// So, now, we can look around
					Case c = map.getCase(nextX, nextY);
					Fish partner = c.getFish();
					if (partner != null && partner.getSex() != sex && partner.canCopulate())
					{
						copulate();
						partner.copulate();
						return;
					}
				}
			}
		}
	}

	public boolean canCopulate()
	{
		return (age >
						lastCopulation + Parameters.FISH_NB_ITERATIONS_ENTRE_COPULATIONS);
	}

	public void copulate()
	{
		lastCopulation = age;
		if (sex == Fish.SEX_FEMALE)
		{
			int babyPerimeter = 5;
			for (int nextX = x - babyPerimeter; nextX < x + babyPerimeter; ++nextX)
			{
				for (int nextY = y - babyPerimeter; nextY < y + babyPerimeter; ++nextY)
				{
					// On vérfie qu'on a des coordonnées valides
					if (map.isCorrectCase(nextX, nextY))
					{
						// So, now, we can look around
						Case c = map.getCase(nextX, nextY);
						if (c.getFish() == null)
						{
							Fish baby = new Fish(map);
							c.setFish(baby);
							baby.setX(nextX);
							baby.setY(nextY);
							return;
						}
					}
				}
			}
		}
	}

	private void eat()
	{
		Case c = map.getCase(x, y);
		int eatable = (int) Math.min(Parameters.FISH_APPETIT, c.getPlanktonQuantity());
		c.setPlanktonQuantity(c.getPlanktonQuantity() - eatable);
		food += eatable;
	}

	private int lookDirection(int dleft, int dtop, int dright, int dbottom)
	{
		int eval = 0;
		for (int i = x - dleft;
						i < x + dright;
						++i)
		{
			for (int j = y - dtop;
							j < y + dbottom;
							++j)
			{
				eval += evalCase(i, j);
			}
		}
		return eval;
	}

	private int evalCase(int i, int j)
	{
		int eval = 0;
		if (map.isCorrectCase(i, j))
		{
			Case caseIJ = map.getCase(i, j);
			if (caseIJ.getShark() != null)
			{

				eval += -(Math.random() * 200);
			}
			eval += caseIJ.getPlanktonQuantity();
			if (caseIJ.getFish() != null)
			{
				int d = (x - i) * (x - i) + (y - j) * (y - j);
				if (d > 5)
				{
					eval += d;
				}
				else
				{
					eval -= d * 6;
				}
			}
			if (caseIJ.getType() == Case.TYPE_PROTECTED)
			{
				eval += 30;
			}
		}
		return eval;
	}

	private void move()
	{

		int directionEval[] = new int[8];

		// On regarde vers l'ouest
		directionEval[0] = lookDirection(Parameters.FISH_CHAMP_VISION,
						Parameters.FISH_CHAMP_VISION / 2, 0, Parameters.FISH_CHAMP_VISION / 2);
		// Sud ouest
		directionEval[1] = lookDirection(Parameters.FISH_CHAMP_VISION,
						0, 0, Parameters.FISH_CHAMP_VISION);
		// sud
		directionEval[2] = lookDirection(Parameters.FISH_CHAMP_VISION / 2,
						0, Parameters.FISH_CHAMP_VISION / 2, Parameters.FISH_CHAMP_VISION);
		// sud est
		directionEval[3] = lookDirection(0,
						0, Parameters.FISH_CHAMP_VISION, Parameters.FISH_CHAMP_VISION);
		// est
		directionEval[4] = lookDirection(0,
						Parameters.FISH_CHAMP_VISION / 2, Parameters.FISH_CHAMP_VISION,
						Parameters.FISH_CHAMP_VISION / 2);
		// nord est
		directionEval[5] = lookDirection(0,
						Parameters.FISH_CHAMP_VISION, Parameters.FISH_CHAMP_VISION,
						0);
		// nord
		directionEval[6] = lookDirection(Parameters.FISH_CHAMP_VISION / 2,
						Parameters.FISH_CHAMP_VISION, Parameters.FISH_CHAMP_VISION / 2,
						0);
		// nord ouest
		directionEval[7] = lookDirection(Parameters.FISH_CHAMP_VISION,
						Parameters.FISH_CHAMP_VISION, 0,
						0);

		// On ajoute une meilleure proba d'aller dans la meme direction
		//directionEval[direction] += 100;
		// On évite de faire demi-tour
		//directionEval[(direction + 4) % 8] -= 100;

		int best = 0;
		for (int i = 1; i < 8; i++)
		{
			if (directionEval[i] > directionEval[best])
			{
				best = i;
			}
		}

		int newX = 0;
		int newY = 0;

		switch (best)
		{
			case 0:
				newX = x - 1;
				newY = y;
				break;
			case 1:
				newX = x - 1;
				newY = y + 1;
				break;
			case 2:
				newX = x;
				newY = y + 1;
				break;
			case 3:
				newX = x + 1;
				newY = y + 1;
				break;
			case 4:
				newX = x + 1;
				newY = y;
				break;
			case 5:
				newX = x + 1;
				newY = y - 1;
				break;
			case 6:
				newX = x;
				newY = y - 1;
				break;
			case 7:
				newX = x - 1;
				newY = y - 1;
				break;
		}



		int placed = 0;
		while (placed < 5)
		{
			if (placed > 1)
			{
				newX = (int) Math.round(x + Math.random() * 2 - 1);
				newY = (int) Math.round(y + Math.random() * 2 - 1);
			}

			if (map.isCorrectCase(newX, newY))
			{

				Case newC = map.getCase(newX, newY);
				// Si la nouvelle case est libre
				if (newC.getFish() == null)
				{
					Case c = map.getCase(x, y);
					c.setFish(null);
					c = map.getCase(newX, newY);
					c.setFish(this);

					x = newX;
					y = newY;

					placed = 10;
				}
			}
			placed++;
		}
	}

	public boolean getSex()
	{
		return sex;
	}

	public static void loadGfx()
	{
		sprite = Toolkit.getDefaultToolkit().getImage(
						Fish.class.getResource("/ecosysteme/resources/fish.png"));
	}
}
