/* Copyright (c) 2009, Guillaume Delhumeau & Sophie Poullain
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Guillaume Delhumeau, Sophie Poullain nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package ecosysteme.model;

/**
 * Les principaux paramètres se trouvent dans cette classe.
 * @author Guillaume Delhumeau & Sophie Poullain
 */
public class Parameters
{

	public static int SAFE_ZONES_NUMBER;
	public static int SAFE_ZONES_SIZE;
	public static int PLANKTON_REGENERATION;
	public static int FISH_SHELF_LIFE;
	public static int FISH_APPETIT;
	public static int FISH_NB_ITERATIONS_ENTRE_COPULATIONS;
	public static int FISH_CHAMP_VISION;
	public static int FISH_FOOD_QUI_SE_PERD;
	public static int SHARK_FOOD_QUI_SE_PERD;
	public static int SHARK_FOOD_MANGE;
	public static int SHARK_FOOD_AVANT_FAIM;
	public static int SHARK_FOOD_POUR_COPULER;
	public static int SHARK_FOOD_COPULATION;
	public static int SHARK_RATISSAGE_EAT;
	public static int SHARK_SHELF_LIFE;
	public static int SHARK_APPETIT;
	public static int SHARK_NB_ITERATIONS_ENTRE_COPULATIONS;
	public static int SHARK_CHAMP_VISION;

	public static void setDefaultValues()
	{
		// Durée de vie du poisson
		FISH_SHELF_LIFE = 5000;
		// Quantité de plancton qu'un poisson mange par itération
		FISH_APPETIT = 10;
		// Durée de vie d'un requin
		SHARK_SHELF_LIFE = 1500;
		// Nombre d'itérations entre 2 reproductions (Poisson)
		FISH_NB_ITERATIONS_ENTRE_COPULATIONS = 30;
		// Nombre d'itérations entre 2 reproductions (Requin)
		SHARK_NB_ITERATIONS_ENTRE_COPULATIONS = 220;
		// Quantité de planction qui peut se regénérer en une itération
		PLANKTON_REGENERATION = 10;
		// Nombre de rochers (protection pour les poissons)
		SAFE_ZONES_NUMBER = 3;
		// Taille des rochers (rayon)
		SAFE_ZONES_SIZE = 18;
		// Quantité de nourriture que le poisson perd de sa réserve par itération
		FISH_FOOD_QUI_SE_PERD = 5;
		// Champ de vision (Poisson)
		FISH_CHAMP_VISION = 9;
		// Champ de vision (Requin)
		SHARK_CHAMP_VISION = 20;
		// Quantité de nourriture que le requin perd de sa réserve par itération
		SHARK_FOOD_QUI_SE_PERD = 1;
		// Quantité de point de nourriture qu'un requin gagne quand il mange un poisson
		SHARK_FOOD_MANGE = 20;
		// Quantité de point de nourriture dont a besoin un requin avant d'être repus
		SHARK_FOOD_AVANT_FAIM = 150;
		// Quantitée de point de nourriture qu'un requin doit avoir pour se reproduire
		SHARK_FOOD_POUR_COPULER = 20;
		// Energie (point de nourriture) consommée pour se reproduire
		SHARK_FOOD_COPULATION = 10;
		// Périmètre des zones d'exploration des requins
		SHARK_RATISSAGE_EAT = 3;
	}
}
