/* Copyright (c) 2009, Guillaume Delhumeau & Sophie Poullain
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Guillaume Delhumeau, Sophie Poullain nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package ecosysteme.model;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;

/**
 * Représente un requin.
 * @author Guillaume Delhumeau & Sophie Poullain
 */
public class Shark
{

	private static Image[] sprite;
	private final static int DIRECTION_E = 0;
	private final static int DIRECTION_SE = 1;
	private final static int DIRECTION_S = 2;
	private final static int DIRECTION_SW = 3;
	private final static int DIRECTION_W = 4;
	private final static int DIRECTION_NW = 5;
	private final static int DIRECTION_N = 6;
	private final static int DIRECTION_NE = 7;
	private int x;
	private int y;
	private int direction;
	private Map map;
	private int age;
	private int lastIter;
	private int food;
	public final static boolean SEX_FEMALE = false;
	public final static boolean SEX_MALE = true;
	private int lastCopulation;
	private boolean sex;
	private static int sharkCounter = 0;
	public int numShark = 0;

	public Shark(Map map)
	{
		this.map = map;
		direction = DIRECTION_SW;
		lastIter = 0;
		lastCopulation = 400;
		numShark = sharkCounter++;
		age = 0;
		food = 100;
		if (Math.random() > 0.5)
		{
			sex = SEX_FEMALE;
		}
		else
		{
			sex = SEX_MALE;
		}
	}

	public int getX()
	{
		return x;
	}

	public void setX(int x)
	{
		this.x = x;
	}

	public int getY()
	{
		return y;
	}

	public void setY(int y)
	{
		this.y = y;
	}

	public void draw(Graphics g)
	{
		//g.setColor(Color.pink);
		//g.fillRect(x * Case.CASE_WIDTH, y * Case.CASE_HEIGHT, Case.CASE_WIDTH, Case.CASE_HEIGHT);
		int spriteX = x * Case.CASE_WIDTH;
		int spriteY = y * Case.CASE_HEIGHT;
		switch (direction)
		{
			case DIRECTION_W:
				spriteX = x * Case.CASE_WIDTH;
				spriteY = y * Case.CASE_HEIGHT - 37;
				break;
			case DIRECTION_NW:
				spriteX = x * Case.CASE_WIDTH - 7;
				spriteY = y * Case.CASE_HEIGHT - 15;
				break;
			case DIRECTION_N:
				spriteX = x * Case.CASE_WIDTH - 50;
				spriteY = y * Case.CASE_HEIGHT + 3;
				break;
			case DIRECTION_NE:
				spriteX = x * Case.CASE_WIDTH - 83;
				spriteY = y * Case.CASE_HEIGHT - 15;
				break;
			case DIRECTION_E:
				spriteX = x * Case.CASE_WIDTH - 93;
				spriteY = y * Case.CASE_HEIGHT - 50;
				break;
			case DIRECTION_SE:
				spriteX = x * Case.CASE_WIDTH - 74;
				spriteY = y * Case.CASE_HEIGHT - 84;
				break;
			case DIRECTION_S:
				spriteX = x * Case.CASE_WIDTH - 40;
				spriteY = y * Case.CASE_HEIGHT - 90;
				break;
			case DIRECTION_SW:
				spriteX = x * Case.CASE_WIDTH - 17;
				spriteY = y * Case.CASE_HEIGHT - 81;
				break;
		}
		g.drawImage(sprite[direction], spriteX, spriteY, null);
	}

	public static void loadGfx()
	{
		sprite = new Image[8];
		sprite[DIRECTION_E] = Toolkit.getDefaultToolkit().getImage(
						Shark.class.getResource("/ecosysteme/resources/shark/SHARK-NORMAL-E.png"));
		sprite[DIRECTION_SE] = Toolkit.getDefaultToolkit().getImage(
						Shark.class.getResource("/ecosysteme/resources/shark/SHARK-NORMAL-SE.png"));
		sprite[DIRECTION_S] = Toolkit.getDefaultToolkit().getImage(
						Shark.class.getResource("/ecosysteme/resources/shark/SHARK-NORMAL-S.png"));
		sprite[DIRECTION_SW] = Toolkit.getDefaultToolkit().getImage(
						Shark.class.getResource("/ecosysteme/resources/shark/SHARK-NORMAL-SW.png"));
		sprite[DIRECTION_W] = Toolkit.getDefaultToolkit().getImage(
						Shark.class.getResource("/ecosysteme/resources/shark/SHARK-NORMAL-W.png"));
		sprite[DIRECTION_NW] = Toolkit.getDefaultToolkit().getImage(
						Shark.class.getResource("/ecosysteme/resources/shark/SHARK-NORMAL-NW.png"));
		sprite[DIRECTION_N] = Toolkit.getDefaultToolkit().getImage(
						Shark.class.getResource("/ecosysteme/resources/shark/SHARK-NORMAL-N.png"));
		sprite[DIRECTION_NE] = Toolkit.getDefaultToolkit().getImage(
						Shark.class.getResource("/ecosysteme/resources/shark/SHARK-NORMAL-NE.png"));


	}    //

	public void update(int iter)
	{
		if (iter > lastIter)
		{
			move();
			eat();
			tryCopulation();
			getOlder();
			lastIter = iter;
		}

	}

	public void getOlder()
	{
		age++;

		if (age > Parameters.SHARK_SHELF_LIFE)
		{
			if (Math.random() * 10000 > 9000)
			{
				Case c = map.getCase(x, y);
				System.out.println("Requin n°" + numShark + " mort de vieillesse");
				c.setShark(null);
			}
		}
		// assez de nourriture?
		if (food <= 0)
		{
			// if (Math.random() * 10000 > 8000) {
			Case c = map.getCase(x, y);
			System.out.println("Requin n°" + numShark + " mort de faim");
			c.setShark(null);
		//}
		}

		food -= Parameters.SHARK_FOOD_QUI_SE_PERD;

	}

	public void move()
	{
		int directionEval[] = new int[8];

		// est
		directionEval[DIRECTION_E] = lookDirection(0,
						Parameters.SHARK_CHAMP_VISION / 2, Parameters.SHARK_CHAMP_VISION,
						Parameters.SHARK_CHAMP_VISION / 2);

		// sud est
		directionEval[DIRECTION_SE] = lookDirection(0,
						0, Parameters.SHARK_CHAMP_VISION, Parameters.SHARK_CHAMP_VISION);

		// sud
		directionEval[DIRECTION_S] = lookDirection(Parameters.SHARK_CHAMP_VISION / 2,
						0, Parameters.SHARK_CHAMP_VISION / 2, Parameters.SHARK_CHAMP_VISION);

		// Sud ouest
		directionEval[DIRECTION_SW] = lookDirection(Parameters.SHARK_CHAMP_VISION,
						0, 0, Parameters.SHARK_CHAMP_VISION);


		// On regarde vers l'ouest
		directionEval[DIRECTION_W] = lookDirection(Parameters.SHARK_CHAMP_VISION,
						Parameters.SHARK_CHAMP_VISION / 2, 0, Parameters.SHARK_CHAMP_VISION / 2);

		// nord ouest
		directionEval[DIRECTION_NW] = lookDirection(Parameters.SHARK_CHAMP_VISION,
						Parameters.SHARK_CHAMP_VISION, 0,
						0);

		// nord
		directionEval[DIRECTION_N] = lookDirection(Parameters.SHARK_CHAMP_VISION / 2,
						Parameters.SHARK_CHAMP_VISION, Parameters.SHARK_CHAMP_VISION / 2,
						0);

		// nord est
		directionEval[DIRECTION_NE] = lookDirection(0,
						Parameters.SHARK_CHAMP_VISION, Parameters.SHARK_CHAMP_VISION,
						0);

		// On privilégie la direction courante
		directionEval[(direction + 4) % 8] -= 30;
		directionEval[direction] += 10;


		int best = 0;
		for (int i = 1; i < 8; i++)
		{
			if (directionEval[i] > directionEval[best])
			{
				best = i;
			}
		}

		int newX = 0;
		int newY = 0;

		direction = best;

		switch (best)
		{
			case DIRECTION_W:
				newX = x - 1;
				newY = y;
				break;
			case DIRECTION_SW:
				newX = x - 1;
				newY = y + 1;
				break;
			case DIRECTION_S:
				newX = x;
				newY = y + 1;
				break;
			case DIRECTION_SE:
				newX = x + 1;
				newY = y + 1;
				break;
			case DIRECTION_E:
				newX = x + 1;
				newY = y;
				break;
			case DIRECTION_NE:
				newX = x + 1;
				newY = y - 1;
				break;
			case DIRECTION_N:
				newX = x;
				newY = y - 1;
				break;
			case DIRECTION_NW:
				newX = x - 1;
				newY = y - 1;
				break;
		}

		if (map.isCorrectCase(newX, newY))
		{

			Case newC = map.getCase(newX, newY);
			if (newC.getType() == Case.TYPE_NORMAL)
			{
				// Si la nouvelle case est libre
				if (newC.getShark() == null)
				{
					Case c = map.getCase(x, y);
					c.setShark(null);
					c = map.getCase(newX, newY);
					c.setShark(this);

					x = newX;
					y = newY;

				// placed = 10;
				}
				else
				{
					direction = (int) ((direction + Math.random() * 7) % 8);
				}
			}
			else
			{
				direction = (int) ((direction + Math.random() * 7) % 8);
			}
		}
		else
		{
			direction = (int) ((direction + Math.random() * 7) % 8);
		}
	}

	private int lookDirection(int dleft, int dtop, int dright, int dbottom)
	{
		int eval = 0;
		for (int i = x - dleft;
						i <
						x + dright;
						++i)
		{
			for (int j = y - dtop;
							j <
							y + dbottom;
							++j)
			{
				eval += evalCase(i, j);
			}

		}
		return eval;
	}

	private int evalCase(int i, int j)
	{
		int eval = 0;
		if (map.isCorrectCase(i, j))
		{
			Case caseIJ = map.getCase(i, j);
			if (caseIJ.getShark() != null)
			{
				if (caseIJ.getShark().getSex() != this.getSex() &&
								canCopulate())
				{
					int d = (x - i) * (x - i) + (y - j) * (y - j);
					if (d > 7)
					{
						eval += d;
					}
					else
					{
						eval -= d * 20;
					}
				}
				if (caseIJ.getShark().getSex() == this.getSex())
				{
					int d = (x - i) * (x - i) + (y - j) * (y - j);
					eval -= d * 20;
				}

			}

			if (caseIJ.getType() == Case.TYPE_PROTECTED)
			{
				eval -= 1;
			}

			if (caseIJ.getFish() != null)
			{
				eval += 1;
			}

		}
		return eval;
	}

	private void eat()
	{

		for (int dx = -Parameters.SHARK_RATISSAGE_EAT; dx <=
						Parameters.SHARK_RATISSAGE_EAT; ++dx)
		{
			for (int dy = -Parameters.SHARK_RATISSAGE_EAT; dy <=
							Parameters.SHARK_RATISSAGE_EAT; ++dy)
			{
				if (x + dx >= 0 && x + dx < Map.NB_COLS - 1 && y + dy >= 0 &&
								y + dy < Map.NB_LINES - 1)
				{
					Case currentCase = map.getCase(x + dx, y + dy);
					if (currentCase.getFish() != null)
					{
						if (food < Parameters.SHARK_FOOD_AVANT_FAIM)
						{
							currentCase.setFish(null);
							food +=
											Parameters.SHARK_FOOD_MANGE;
						}

					}
				}
			}
		}
	}

	private void tryCopulation()
	{
		if (!canCopulate())
		{
			return;
		}

		// He can do it !!!! But only if he find someone
		int tailleRatissage = 4;

		// Looking for a partner
		for (int nextX = x - tailleRatissage; nextX <
						x + tailleRatissage; ++nextX)
		{
			for (int nextY = y - tailleRatissage; nextY <
							y + tailleRatissage; ++nextY)
			{
				// On vérfie qu'on a des coordonnées valides
				if (nextX >= 0 && nextX < Map.NB_COLS &&
								nextY >= 0 && nextY < Map.NB_LINES &&
								!(nextX == x && nextY == y))
				{
					// So, now, we can look around
					Case c = map.getCase(nextX, nextY);
					Shark partner = c.getShark();
					if (partner != null && partner.getSex() != sex && partner.canCopulate())
					{
						copulate();
						partner.copulate();
						return;

					}


				}
			}
		}
	}

	public boolean canCopulate()
	{
		return (age >
						lastCopulation + Parameters.SHARK_NB_ITERATIONS_ENTRE_COPULATIONS &&
						food > Parameters.SHARK_FOOD_POUR_COPULER);

	}

	public void copulate()
	{
		lastCopulation = age;
		if (sex == Shark.SEX_FEMALE)
		{
			for (int nextX = x - 4; nextX <
							x + 4; ++nextX)
			{
				for (int nextY = y - 4; nextY <
								y + 4; ++nextY)
				{
					// On vérfie qu'on a des coordonnées valides
					if (map.isCorrectCase(nextX, nextY) &&
									map.getCase(nextX, nextY).getType() != Case.TYPE_PROTECTED)
					{
						// So, now, we can look around
						Case c = map.getCase(nextX, nextY);
						if (c.getShark() == null)
						{
							Shark baby = new Shark(map);
							c.setShark(baby);
							baby.setX(nextX);
							baby.setY(nextY);
							food -= Parameters.SHARK_FOOD_COPULATION;
							return;

						}


					}
				}
			}
		}
	}

	public boolean getSex()
	{
		return sex;
	}

	public int getLastCopulation()
	{
		return lastCopulation;
	}

	public void setLastCopulation(int lastCopulation)
	{
		this.lastCopulation = lastCopulation;
	}
}
